#!/bin/bash
#
# This script is used to compile and run the provided CM testbench
# Please see bottom of the case statement for usage
#
# EDA simulator setup must be done including license setup
#

MODEL_NAME="twocounter"
TB_NAME="tb"

# 1st argument is the simulation to run
sim=$1
shift 1

#pass other arguments to the simulator
if [ "$#" -gt 0 ]; then
    PLUSARGS=$*
fi

if [[ ! $MODEL_PATH ]]
then
  export TB_PATH=`dirname \`which $0\``
  export ROOT_PATH=`dirname $TB_PATH`
  export MODEL_PATH=$ROOT_PATH/cm
fi

case $sim in

    vcs-hdl)
        echo "Running original HDL only simulation in VCS"

        vcs -full64 -debug_access+r ${MODEL_PATH}/${MODEL_NAME}.v ${TB_NAME}.v
        ./simv ${PLUSARGS} 
        exit $?
        ;;

    vcs )
        echo "Running in VCS with cycle model"

        syscan -full64 -sysc=231 -cflags "-std=c++11 -I${MODEL_PATH}/ -I${MODEL_PATH}/gcc -I${CARBON_HOME}/include" ${MODEL_PATH}/${MODEL_NAME}.cpp:${MODEL_NAME} |& tee vcs.log 

        vcs -full64 -debug_access+r -sysc -sysc=231 -sysc=adjust_timeres -l vcs.log -cflags "-std=c++11 -I${MODEL_PATH}" -LDFLAGS "-L${MODEL_PATH}" ${TB_NAME}.v ${CARBON_HOME}/Linux64/lib/gcc/shared/libcarbon5.so ${MODEL_PATH}/gcc/lib${MODEL_NAME}.a |& tee -a vcs.log 

        ./simv ${PLUSARGS} 

        exit $?
        ;;

    ius-hdl )
        echo "Running original HDL only simulation in ius"

        irun -64bit +access+r -64bit ${MODEL_PATH}/${MODEL_NAME}.v ${TB_NAME}.v ${PLUSARGS}

        exit $?
        ;;

    ius )
        echo "Running in ius with cycle model"

        irun +access+r -64bit -Wcxx,-std=c++11 -sysc -scautoshell verilog -DCARBON_DUMP_VCD=1 -I${MODEL_PATH} -I${MODEL_PATH}/gcc -I${CARBON_HOME}/include -L${MODEL_PATH} ${MODEL_PATH}/${MODEL_NAME}.cpp -L${CARBON_HOME}/Linux64/lib/gcc/shared -lcarbon5 ${MODEL_PATH}/gcc/lib${MODEL_NAME}.a ${TB_NAME}.v ${PLUSARGS}

        exit $?
        ;;

    mti-hdl )
        echo "Running original HDL only simulation in mti"

        vlog -64 ${MODEL_PATH}/${MODEL_NAME}.v ${TB_NAME}.v 

        if [[ $PLUSARGS = *"-gui"* ]]; then
            vsim -64 ${TB_NAME} ${PLUSARGS}
        else
            vsim -64 -c -do "run -all" ${TB_NAME} ${PLUSARGS}
        fi

        exit $?
        ;;

    mti )
        echo "Running in mti with cycle model"

        vlog -64 ${TB_NAME}.v 

        sccom -64  -g -suppress 6102 -I${MODEL_PATH} -I${MODEL_PATH}/gcc -I${CARBON_HOME}/include ${MODEL_PATH}/${MODEL_NAME}.cpp 

        sccom -64 -link -suppress 6102 -L${MODEL_PATH}/gcc -l${MODEL_NAME} -L ${CARBON_HOME}/Linux64/lib/gcc/shared -lcarbon5 

        if [[ $PLUSARGS = *"-gui"* ]]; then
            vsim -64  ${TB_NAME} ${PLUSARGS}
        else
            vsim -64 -c -do "run -all" ${TB_NAME} ${PLUSARGS}
        fi

        exit $?
        ;;
    clean )
        echo Removing old build and run output
        rm -rf *.log *.history *.o AN.DB csrc INCA_libs simv* work ucli.key transcript *.vcd .simvision DVEfiles *.vpd irun.key *.dsn *.trn
        ;;
    *)
        echo
        echo "Incorrect usage $1"
        echo
        echo "$0 {vcs|ius|mti|vcs-hdl|ius-hdl|mti-hdl} [options]"
        echo "Run the testbench simulation"
        echo "  vcs-hdl:  VCS simulator with original HDL"
        echo "  vcs:      VCS simulator with cycle model"
        echo "  ius-hdl:  IUS simulator with original HDL"
        echo "  ius:      IUS simulator with cycle model"
        echo "  mti-hdl:  QuestaSim simulator with original HDL"
        echo "  mti:      QuestaSim simulator with cycle model"
        echo "  clean:    delete simulator artifacts"
        ;;
esac
